@@ -170,12 +170,15 @@ static GList *event_listeners = NULL;
 static gchar *
 convert_name_from_dbus (const char *name, gboolean path_hack)
 {
-  gchar *ret = g_malloc (g_utf8_strlen (name, -1) * 2 + 1);
+  gchar *ret;
   const char *p = name;
-  gchar *q = ret;
+  gchar *q;
 
-  if (!ret)
-    return NULL;
+  if (!name)
+    return g_strdup ("");
+
+ ret = g_malloc (g_utf8_strlen (name, -1) * 2 + 1);
+  q = ret;
 
   while (*p)
   {
@@ -379,24 +382,32 @@ convert_event_type_to_dbus (const char *eventType, char **categoryp, char **name
   if (matchrule_array)
   {
     gchar *matchrule;
-    matchrule = g_strdup_printf ("type='signal',interface='org.a11y.atspi.Event.%s'", category);
-    if (name && name [0])
-    {
-      gchar *new_str = g_strconcat (matchrule, ",member='", name, "'", NULL);
-      g_free (matchrule);
-      matchrule = new_str;
-    }
     (*matchrule_array) = g_ptr_array_new ();
-    if (detail && detail [0])
+    if (!strcmp (eventType, "object:text-reading-position"))
     {
-      gchar *new_str = g_strconcat (matchrule, ",arg0='", detail, "'", NULL);
-      g_ptr_array_add (*matchrule_array, new_str);
-      new_str = g_strconcat (matchrule, ",arg0path='", detail, "/'", NULL);
-      g_ptr_array_add (*matchrule_array, new_str);
-      g_free (matchrule);
+      matchrule = g_strdup ("type='signal',interface='org.a11y.atspi.ScreenReader',member='ReadingPosition',sender='org.a11y.Atspi.ScreenReader'");
+      g_ptr_array_add (*matchrule_array, matchrule);
     }
     else
-      g_ptr_array_add (*matchrule_array, matchrule);
+    {
+    matchrule = g_strdup_printf ("type='signal',interface='org.a11y.atspi.Event.%s'", category);
+      if (name && name [0])
+      {
+        gchar *new_str = g_strconcat (matchrule, ",member='", name, "'", NULL);
+        g_free (matchrule);
+        matchrule = new_str;
+      }
+      if (detail && detail [0])
+      {
+        gchar *new_str = g_strconcat (matchrule, ",arg0='", detail, "'", NULL);
+        g_ptr_array_add (*matchrule_array, new_str);
+        new_str = g_strconcat (matchrule, ",arg0path='", detail, "/'", NULL);
+        g_ptr_array_add (*matchrule_array, new_str);
+        g_free (matchrule);
+      }
+      else
+        g_ptr_array_add (*matchrule_array, matchrule);
+    }
   }
   if (categoryp) *categoryp = category;
   else g_free (category);
@@ -459,6 +470,7 @@ listener_entry_free (EventListenerEntry *e)
  *            object:text-selection-changed
  *            object:text-changed
  *            object:text-caret-moved
+ *            object:text-reading-position
  *            object:row-inserted
  *            object:row-reordered
  *            object:row-deleted
@@ -507,6 +519,9 @@ listener_entry_free (EventListenerEntry *e)
  *            In general, listening to
  *            toolkit-specific events is not recommended.
  *
+ * Currently, object:text-reading-position needs to be specified explicitly
+ * (it is not implied by object:text), since it is generated by the screen
+ * reader and is thus a special case internally.
  *
  * Returns: #TRUE if successful, otherwise #FALSE.
  **/
@@ -962,11 +977,22 @@ _atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
   dbus_int32_t detail1, detail2;
   char *p;
   GHashTable *cache = NULL;
+  gboolean is_reading_position = 0;
 
-  if (strcmp (signature, "siiv(so)") != 0 &&
+  if (dbus_message_is_signal (message, ATSPI_DBUS_INTERFACE_SCREEN_READER,
+      "ReadingPosition"))
+  {
+    if (strcmp (signature, "(so)ii") != 0)
+    {
+      g_warning ("Got invalid signature '%s' for ReadingPosition signal", signature);
+      return DBUS_HANDLER_RESULT_HANDLED;
+    }
+    is_reading_position = TRUE;
+  }
+  else if (strcmp (signature, "siiv(so)") != 0 &&
       strcmp (signature, "siiva{sv}") != 0)
   {
-    g_warning ("Got invalid signature %s for signal %s from interface %s\n", signature, member, category);
+   g_warning ("Got invalid signature %s for signal %s from interface %s\n", signature, member, category);
     return DBUS_HANDLER_RESULT_HANDLED;
   }
 
@@ -981,9 +1007,16 @@ _atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
       return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
     category++;
+    if (!strcmp (category, "ScreenReader"))
+      category = "Object"; /* hack -- assume this is ReadingPosition */
+  }
+  if (is_reading_position)
+    e.source = _atspi_dbus_return_accessible_from_iter (&iter);
+  else
+  {
+    dbus_message_iter_get_basic (&iter, &detail);
+    dbus_message_iter_next (&iter);
   }
-  dbus_message_iter_get_basic (&iter, &detail);
-  dbus_message_iter_next (&iter);
   dbus_message_iter_get_basic (&iter, &detail1);
   e.detail1 = detail1;
   dbus_message_iter_next (&iter);
@@ -992,7 +1025,10 @@ _atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
   dbus_message_iter_next (&iter);
 
   converted_type = convert_name_from_dbus (category, FALSE);
-  name = convert_name_from_dbus (member, FALSE);
+  if (is_reading_position)
+    name = g_strdup ("text-reading-position");
+  else
+    name = convert_name_from_dbus (member, FALSE);
   detail = convert_name_from_dbus (detail, TRUE);
 
   if (strcasecmp  (category, name) != 0)
@@ -1015,54 +1051,58 @@ _atspi_dbus_handle_event (DBusConnection *bus, DBusMessage *message, void *data)
     converted_type = p;
   }
   e.type = converted_type;
-  e.source = _atspi_ref_accessible (dbus_message_get_sender(message), dbus_message_get_path(message));
+  if (!is_reading_position)
+    e.source = _atspi_ref_accessible (dbus_message_get_sender(message), dbus_message_get_path(message));
   if (e.source == NULL)
   {
-    g_warning ("Got no valid source accessible for signal for signal %s from interface %s\n", member, category);
+    g_warning ("Got no valid source accessible for signal %s from interface %s\n", member, category);
     g_free (converted_type);
     g_free (name);
     g_free (detail);
     return DBUS_HANDLER_RESULT_HANDLED;
   }
 
-  dbus_message_iter_recurse (&iter, &iter_variant);
-  switch (dbus_message_iter_get_arg_type (&iter_variant))
+  if (!is_reading_position)
   {
-    case DBUS_TYPE_STRUCT:
+    dbus_message_iter_recurse (&iter, &iter_variant);
+    switch (dbus_message_iter_get_arg_type (&iter_variant))
     {
-      AtspiRect rect;
-      if (demarshal_rect (&iter_variant, &rect))
+      case DBUS_TYPE_STRUCT:
       {
-	g_value_init (&e.any_data, ATSPI_TYPE_RECT);
-	g_value_set_boxed (&e.any_data, &rect);
+        AtspiRect rect;
+        if (demarshal_rect (&iter_variant, &rect))
+        {
+          g_value_init (&e.any_data, ATSPI_TYPE_RECT);
+          g_value_set_boxed (&e.any_data, &rect);
+        }
+        else
+        {
+          AtspiAccessible *accessible;
+	  accessible = _atspi_dbus_return_accessible_from_iter (&iter_variant);
+          g_value_init (&e.any_data, ATSPI_TYPE_ACCESSIBLE);
+          g_value_set_instance (&e.any_data, accessible);
+          if (accessible)
+            g_object_unref (accessible);	/* value now owns it */
+        }
+        break;
       }
-      else
+      case DBUS_TYPE_STRING:
       {
-        AtspiAccessible *accessible;
-	accessible = _atspi_dbus_return_accessible_from_iter (&iter_variant);
-	g_value_init (&e.any_data, ATSPI_TYPE_ACCESSIBLE);
-	g_value_set_instance (&e.any_data, accessible);
-	if (accessible)
-	  g_object_unref (accessible);	/* value now owns it */
+        dbus_message_iter_get_basic (&iter_variant, &p);
+        g_value_init (&e.any_data, G_TYPE_STRING);
+        g_value_set_string (&e.any_data, p);
+        break;
       }
+    default:
       break;
     }
-    case DBUS_TYPE_STRING:
-    {
-      dbus_message_iter_get_basic (&iter_variant, &p);
-      g_value_init (&e.any_data, G_TYPE_STRING);
-      g_value_set_string (&e.any_data, p);
-      break;
-    }
-  default:
-    break;
-  }
 
-  dbus_message_iter_next (&iter);
-  if (dbus_message_iter_get_arg_type (&iter) == DBUS_TYPE_ARRAY)
-  {
-    /* new form -- parse properties sent with event */
-    cache = _atspi_dbus_update_cache_from_dict (e.source, &iter);
+    dbus_message_iter_next (&iter);
+    if (dbus_message_iter_get_arg_type (&iter) == DBUS_TYPE_ARRAY)
+    {
+      /* new form -- parse properties sent with event */
+      cache = _atspi_dbus_update_cache_from_dict (e.source, &iter);
+    }
   }
 
   if (!strncmp (e.type, "object:children-changed", 23))
